"""Register all builtin tasks."""
import gymnasium as gym

from angorapy.common.const import N_SUBSTEPS
from angorapy.common.const import SHADOWHAND_MAX_STEPS
from angorapy.tasks.envs.cognitive.hanoi import HanoiEnv
from angorapy.tasks.envs.dexterity.manipulate import ManipulateBlock
from angorapy.tasks.envs.dexterity.manipulate import ManipulateBlockAsymmetric
from angorapy.tasks.envs.dexterity.manipulate import ManipulateBlockDiscrete
from angorapy.tasks.envs.dexterity.manipulate import ManipulateBlockDiscrete
from angorapy.tasks.envs.dexterity.manipulate import ManipulateBlockDiscreteAsymmetric
from angorapy.tasks.envs.dexterity.reach import FreeReach
from angorapy.tasks.envs.dexterity.reach import FreeReachSequential
from angorapy.tasks.envs.dexterity.reach import Reach
from angorapy.tasks.envs.dexterity.reach import ReachSequential

# REACHING
for vision_mode in ["Visual", ""]:
    for control_mode in ["Relative", "Absolute"]:
        gym.envs.register(
            id=f'Reach{control_mode}{vision_mode}-v0',
            entry_point='angorapy.tasks.envs.dexterity.reach:Reach',
            kwargs={"relative_control": control_mode == "Relative", "vision": vision_mode == "Visual",
                    "n_substeps": N_SUBSTEPS},
            max_episode_steps=SHADOWHAND_MAX_STEPS,
        )

        gym.envs.register(
            id=f'Reach{control_mode}{vision_mode}NoTouch-v0',
            entry_point='angorapy.tasks.envs.dexterity.reach:Reach',
            kwargs={"relative_control": control_mode == "Relative", "vision": vision_mode == "Visual", "touch": False,
                    "n_substeps": N_SUBSTEPS},
            max_episode_steps=SHADOWHAND_MAX_STEPS,
        )

        gym.envs.register(
            id=f'FreeReach{control_mode}{vision_mode}-v0',
            entry_point='angorapy.tasks.envs.dexterity.reach:FreeReach',
            kwargs={"relative_control": control_mode == "Relative", "vision": vision_mode == "Visual",
                    "n_substeps": N_SUBSTEPS},
            max_episode_steps=SHADOWHAND_MAX_STEPS,
        )

        for i, name in enumerate(["FF", "MF", "RF", "LF"]):
            gym.envs.register(
                id=f'FreeReach{name}{control_mode}{vision_mode}-v0',
                entry_point='angorapy.tasks.envs.dexterity.reach:FreeReach',
                kwargs={"relative_control": control_mode == "Relative",
                        "vision": vision_mode == "Visual",
                        "force_finger": i,
                        "n_substeps": N_SUBSTEPS},
                max_episode_steps=SHADOWHAND_MAX_STEPS,
            )

            gym.envs.register(
                id=f'Reach{name}{control_mode}{vision_mode}-v0',
                entry_point='angorapy.tasks.envs.dexterity.reach:Reach',
                kwargs={"relative_control": control_mode == "Relative",
                        "vision": vision_mode == "Visual",
                        "force_finger": i,
                        "n_substeps": N_SUBSTEPS},
                max_episode_steps=SHADOWHAND_MAX_STEPS,
            )

# MANIPULATE
gym.envs.register(
    id=f'ManipulateBlock-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlock',
    kwargs={"delta_t": 0.008},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockVisual-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlock',
    kwargs={"delta_t": 0.008, "vision": True},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockDiscrete-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockDiscrete',
    kwargs={"delta_t": 0.008},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockVisualDiscrete-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockDiscrete',
    kwargs={"delta_t": 0.008, "vision": True},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockDiscreteAsymmetric-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockDiscreteAsymmetric',
    kwargs={"delta_t": 0.008},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockVisualDiscreteAsymmetric-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockDiscreteAsymmetric',
    kwargs={"delta_t": 0.008, "vision": True},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockAsymmetric-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockAsymmetric',
    kwargs={"delta_t": 0.008},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockVisualAsymmetric-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockAsymmetric',
    kwargs={"delta_t": 0.008, "vision": True},
    max_episode_steps=50 * 100,
)

# for backwards compatibility; TODO remove at 1.0
gym.envs.register(
    id=f'ManipulateBlockDiscreteAsynchronous-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockDiscreteAsymmetric',
    kwargs={"delta_t": 0.008},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockVisualDiscreteAsynchronous-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockDiscreteAsymmetric',
    kwargs={"delta_t": 0.008, "vision": True},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockAsynchronous-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockAsymmetric',
    kwargs={"delta_t": 0.008},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id=f'ManipulateBlockVisualAsynchronous-v0',
    entry_point='angorapy.tasks.envs.dexterity.manipulate:ManipulateBlockAsymmetric',
    kwargs={"delta_t": 0.008, "vision": True},
    max_episode_steps=50 * 100,
)

gym.envs.register(
    id="HanoiTower-v0",
    entry_point="angorapy.tasks.envs.cognitive.hanoi:HanoiEnv",
)



"""Module for additional environments as well as registering modified environments."""
from angorapy.tasks import envs
from angorapy.tasks import world_building
from angorapy.tasks.core import AnthropomorphicEnv

__all__ = [
    "AnthropomorphicEnv",
    "world_building",
]



import random
import unittest

import numpy as np

from angorapy.tasks.registration import make_task
from angorapy.common.senses import Sensation
from angorapy.common.postprocessors import RewardNormalizer, merge_postprocessors, \
    StateNormalizer
from angorapy.utilities.core import env_extract_dims


def test_state_normalization():
    env_name = "ManipulateBlockDiscreteAsynchronous-v0"
    env = make_task(env_name)
    normalizer = StateNormalizer(env_name, *env_extract_dims(env))

    env.reset()
    inputs = [env.step(env.action_space.sample())[0] for _ in range(150)]

    for sample in inputs:
        o, _, _, _, _ = normalizer.transform((sample, 1.0, False, False, {}))

    true_mean = np.sum(inputs, axis=0) / len(inputs)
    true_std = np.mean([(i - Sensation(**normalizer.mean)) ** 2 for i in inputs])

    for name in true_mean.dict().keys():
        assert np.allclose(true_mean[name], normalizer.mean[name], atol=1e-6), f"{name}'s mean not equal."
        assert np.allclose(true_std[name], normalizer.variance[name], atol=1e-5), f"{name}'s std not equal."


def test_state_normalization_non_anthropomorphic():
    env_name = "LunarLanderContinuous-v2"
    env = make_task(env_name)
    normalizer = StateNormalizer(env_name, *env_extract_dims(env))

    env.reset()
    inputs = [env.step(env.action_space.sample())[0] for _ in range(150)]
    true_mean = np.sum(inputs, axis=0) / len(inputs)
    true_std = np.mean([(i - true_mean) ** 2 for i in inputs])

    for sample in inputs:
        o, _, _, _, _ = normalizer.transform((sample, 1.0, False, False, {}))

    for name in true_mean.dict().keys():
        assert np.allclose(true_mean[name], normalizer.mean[name], atol=1e-6), f"{name}'s mean not equal."
        assert np.allclose(true_std[name], normalizer.variance[name], atol=1e-5), f"{name}'s std not equal."


def test_state_normalization_adding():
    env_name = "LunarLanderContinuous-v2"
    env = make_task(env_name)
    normalizer_a = StateNormalizer(env_name, *env_extract_dims(env))
    normalizer_b = StateNormalizer(env_name, *env_extract_dims(env))
    normalizer_c = StateNormalizer(env_name, *env_extract_dims(env))

    inputs_a = [env.observation_space.sample() for _ in range(100)]
    inputs_b = [env.observation_space.sample() for _ in range(100)]
    inputs_c = [env.observation_space.sample() for _ in range(100)]

    true_mean = np.mean(inputs_a + inputs_b + inputs_c, axis=0)
    true_std = np.std(inputs_a + inputs_b + inputs_c, axis=0)

    for sample in inputs_a:
        normalizer_a.update({"proprioception": sample})

    for sample in inputs_b:
        normalizer_b.update({"proprioception": sample})

    for sample in inputs_c:
        normalizer_c.update({"proprioception": sample})

    combined_normalizer = normalizer_a + normalizer_b + normalizer_c
    merged_normalizer = merge_postprocessors([normalizer_a, normalizer_b, normalizer_c])

    assert np.allclose(true_mean, combined_normalizer.mean["proprioception"], atol=1e-6)
    assert np.allclose(true_mean, merged_normalizer.mean["proprioception"], atol=1e-6)
    assert np.allclose(true_std, np.sqrt(combined_normalizer.variance["proprioception"]), atol=1e-6)


def test_reward_normalization_adding():
    env_name = "LunarLanderContinuous-v2"
    env = make_task(env_name)
    normalizer_a = RewardNormalizer(env_name, *env_extract_dims(env))
    normalizer_b = RewardNormalizer(env_name, *env_extract_dims(env))
    normalizer_c = RewardNormalizer(env_name, *env_extract_dims(env))

    inputs_a = [random.random() * 10 for _ in range(1000)]
    inputs_b = [random.random() * 20 for _ in range(1000)]
    inputs_c = [random.random() * 5 for _ in range(1000)]

    true_mean = np.mean(inputs_a + inputs_b + inputs_c, axis=0)
    true_std = np.std(inputs_a + inputs_b + inputs_c, axis=0)

    for sample in inputs_a:
        normalizer_a.update({"reward": sample})

    for sample in inputs_b:
        normalizer_b.update({"reward": sample})

    for sample in inputs_c:
        normalizer_c.update({"reward": sample})

    combined_normalizer = normalizer_a + normalizer_b + normalizer_c

    assert np.allclose(true_mean, combined_normalizer.mean["reward"])
    assert np.allclose(true_std, np.sqrt(combined_normalizer.variance["reward"]))



import abc
import pathlib
from typing import Optional, Sequence, Union

from dm_control import mjcf


class _Entity(abc.ABC):

    def __init__(self,
                 root: mjcf.RootElement,
                 name: str,
                 position: Optional[Sequence[float]] = (0, 0, 0),
                 rotation: Optional[Sequence[float]] = (0, 0, 0, 0)):
        self._mjcf_root = root
        self._mjcf_root.model = name

        self._parse_entity()
        self._setup_entity()

        if position is not None and self.root_body is not None:
            self.root_body.pos = position

        if rotation is not None and self.root_body is not None:
            self.root_body.quat = rotation

    @abc.abstractmethod
    def _parse_entity(self) -> None:
        ...

    @abc.abstractmethod
    def _setup_entity(self) -> None:
        ...

    @property
    def mjcf_model(self) -> mjcf.RootElement:
        return self._mjcf_root

    @property
    def name(self) -> str:
        return self._mjcf_root.model

    @property
    @abc.abstractmethod
    def root_body(self) -> mjcf.Element:
        ...

    def attach(self, other: Union["_Entity", mjcf.Element]) -> mjcf.Element:
        """Attaches another entity to this one."""
        return self.mjcf_model.attach(other.mjcf_model)


class Robot(_Entity):

    def __init__(self,
                 xml_or_mjcf: Union[str, mjcf.RootElement],
                 position: Sequence[float] = (.0, .0, .0),
                 rotation: Sequence[float] = (.0, .0, .0, 0.)):
        if isinstance(xml_or_mjcf, (str, pathlib.Path)):
            mjfc_element = mjcf.from_path(str(xml_or_mjcf))
        elif isinstance(xml_or_mjcf, mjcf.RootElement):
            mjfc_element = xml_or_mjcf
        else:
            raise ValueError(f"xml_or_mjcf must be either a string or a mjcf.RootElement, got {type(xml_or_mjcf)}")

        super().__init__(mjfc_element, "robot", position=position, rotation=rotation)

    @property
    @abc.abstractmethod
    def joints(self) -> Sequence[mjcf.Element]:
        ...

    @property
    @abc.abstractmethod
    def actuators(self) -> Sequence[mjcf.Element]:
        ...


class External(_Entity):

    def __init__(self,
                 root: mjcf.RootElement,
                 name: str,
                 position: Optional[Sequence[float]] = (0, 0, 0),
                 rotation: Optional[Sequence[float]] = (0, 0, 0, 0)):
        super().__init__(root, name, position=position, rotation=rotation)

    @property
    @abc.abstractmethod
    def joints(self) -> Sequence[mjcf.Element]:
        ...


class Stage(_Entity):
    """Stage with lighting."""

    def __init__(self) -> None:
        """Initializes a stage."""
        super().__init__(mjcf.RootElement(), "stage", position=None, rotation=None)

    def _parse_entity(self) -> None:
        pass

    def _setup_entity(self) -> None:
        # Change free camera settings.
        self._mjcf_root.statistic.extent = 0.5
        self._mjcf_root.statistic.center = (0.3, 0, 0.0)
        getattr(self._mjcf_root.visual, "global").azimuth = 220
        getattr(self._mjcf_root.visual, "global").elevation = -30

        self._mjcf_root.visual.scale.forcewidth = 0.04
        self._mjcf_root.visual.scale.contactwidth = 0.2
        self._mjcf_root.visual.scale.contactheight = 0.03

        # Lights.
        self._mjcf_root.worldbody.add("light", pos=(0, 0, 1))
        self._mjcf_root.worldbody.add(
            "light", pos=(0.3, 0, 1.5), dir=(0, 0, -1), directional=True
        )

        # Dark checkerboard floor.
        self._mjcf_root.asset.add(
            "texture",
            name="grid",
            type="2d",
            builtin="checker",
            mark="cross",
            width=256,
            height=256,
            # rgb1=[0.2, 0.3, 0.4],
            # rgb2=[0.1, 0.2, 0.3],
            rgb1=[1, 1, 1],
            rgb2=[1, 1, 1],

            markrgb=[0.8, 0.8, 0.8],
        )
        self._mjcf_root.asset.add(
            "material",
            name="grid",
            texture="grid",
            texrepeat=(5, 5),
            texuniform=True,
            reflectance=0.2,
        )
        self._ground_geom = self._mjcf_root.worldbody.add(
            "geom",
            type="plane",
            size=(0, 0, 0.05),
            pos=(0, 0, -0.1),
            material="grid",
            contype=0,
            conaffinity=0,
        )

        self._mjcf_root.asset.add(
            "texture",
            name="skybox",
            type="skybox",
            builtin="gradient",
            rgb1=[0.2, 0.2, 0.2],
            rgb2=[0.0, 0.0, 0.0],
            width=800,
            height=800,
            mark="random",
            markrgb=[1, 1, 1],
        )

    @property
    def root_body(self) -> mjcf.Element:
        return self.mjcf_model.find("worldbody")



#!/usr/bin/env python
"""Core methods providing functionality to the agent."""
from itertools import accumulate
from typing import List

import numpy as np
import tensorflow as tf
from scipy.signal import lfilter

from angorapy.common.const import NP_FLOAT_PREC


def get_discounted_returns(reward_trajectory, discount_factor: float):
    """Discounted future rewards calculation using itertools. Way faster than list comprehension."""
    return list(accumulate(reward_trajectory[::-1], lambda previous, x: previous * discount_factor + x))[::-1]


def estimate_advantage(rewards: List, values: List, t_is_terminal: List, gamma: float, lam: float) -> np.ndarray:
    """K-Step return estimator for Generalized Advantage Estimation.
    From: HIGH-DIMENSIONAL CONTINUOUS CONTROL USING GENERALIZED ADVANTAGE ESTIMATION. (Schulman et. al., 2018)

    Using the raw discounted future reward suffers from high variance and hence could hinder learning. Using a k-step
    return estimator introduces some bias but reduces variance, yielding a beneficial trade-off.

    :param rewards:             list of rewards where r[t] is the reward from taking a[t] in state serialization[t],
                                transitioning to serialization[t + 1]
    :param values:              value estimations for state trajectory. Requires a value for last state not covered in
                                rewards, too, as this might be non-terminal
    :param t_is_terminal        boolean indicator list of false for non-terminal and true for terminal states. A
                                terminal state is one after which the absorbing state follows and the episode ends.
                                Hence if t is terminal, this is not the last state observed but the last in which
                                experience can be collected through taking an step_tuple.
    :param gamma:               a discount factor weighting the importance of future rewards
    :param lam:                 GAE'serialization lambda parameter compromising between bias and variance. High lambda
                                results in
                                less bias but more variance. 0 < Lambda < 1

    :return:                    the estimations about the returns of a trajectory
    """
    if np.size(rewards, 0) - np.size(values, 0) != -1:
        raise ValueError("Values must include one more prediction than there are rewards.")

    total_steps = np.size(rewards, 0)
    return_estimations = np.ndarray(shape=(total_steps,)).astype(np.float32)

    previous = 0
    for t in reversed(range(total_steps)):
        if t_is_terminal[t]:
            previous = 0

        delta = rewards[t] - values[t]
        if not t_is_terminal[t]:
            delta += (gamma * values[t + 1])
        previous = delta + gamma * lam * previous

        return_estimations[t] = previous

    return return_estimations


def estimate_episode_advantages(rewards, values, discount, lam):
    """Estimate advantage of a single episode (or part of it).

    Taken from Open AI'serialization spinning up repository."""
    deltas = np.array(rewards) + discount * np.array(values[1:]) - np.array(values[:-1])
    return lfilter([1], [1, float(-(discount * lam))], deltas[::-1], axis=0)[::-1].astype(NP_FLOAT_PREC)


@tf.function
def extract_discrete_action_probabilities(predictions: tf.Tensor, actions: tf.Tensor) -> tf.Tensor:
    """Given a tensor of predictions with shape [batch_size, sequence, n_actions] or [batch_size, n_actions] and a 2D or
    1D tensor of actions with shape [batch_size, sequence_length] or [batch_size] extract the probabilities for the
    actions."""
    assert len(predictions.shape) in [2, 3], "Predictions should be a tensor of rank 2 or 3."
    assert len(actions.shape) in [1, 2], "Actions should be a tensor of rank 1 or 2."

    if len(actions.shape) == 1:
        indices = tf.concat([tf.reshape(tf.range(actions.shape[0]), [-1, 1]), tf.reshape(actions, [-1, 1])], axis=-1)
        choices = tf.gather_nd(predictions, indices)
    else:
        batch_indices = tf.reshape(
            tf.tile(tf.expand_dims(tf.range(actions.shape[0]), axis=-1), [1, actions.shape[1]]), [-1, 1])
        sequence_indices = tf.reshape(
            tf.tile(tf.expand_dims(tf.range(actions.shape[1]), axis=0), [1, actions.shape[0]]), [-1, 1])

        indices = tf.concat((batch_indices, sequence_indices, tf.reshape(actions, [-1, 1])), axis=-1)

        choices = tf.gather_nd(predictions, indices)
        choices = tf.reshape(choices, actions.shape)

    return choices



import numpy as np
import tensorflow as tf

from angorapy import make_task
from angorapy.common.policies import BetaPolicyDistribution
from angorapy.utilities.model_utils import reset_states_masked, reset_states_masked_tf
from angorapy.utilities.core import flatten
from angorapy.models import get_model_builder


def test_model_state_reset():
    env = make_task("LunarLanderContinuous-v2")
    model_builder = get_model_builder("simple", model_type="lstm")
    model, _, _ = model_builder(env, BetaPolicyDistribution(env), bs=5)

    # copy of model with same weights
    model_copy = tf.keras.models.clone_model(model)
    model_copy.set_weights(model.get_weights())

    model(tf.ones((5, 1, 8)))
    model_copy(tf.ones((5, 1, 8)))

    # store copy of the states of the models
    states = [tf.identity(s) for s in flatten([l.states for l in model.layers if hasattr(l, "states")])]
    states_copy = [tf.identity(s) for s in flatten([l.states for l in model_copy.layers if hasattr(l, "states")])]

    # assert whether the states of the two models are equal
    assert np.all([np.all(s == sc) for s, sc in zip(states, states_copy)])

    recurrent_layers = [layer for layer in model.submodules if isinstance(layer, tf.keras.layers.RNN)]
    recurrent_layers_of_copy = [layer for layer in model_copy.submodules if isinstance(layer, tf.keras.layers.RNN)]

    # reset the states of the models
    for _ in range(10):
        for i in range(3):
            model(tf.ones((5, 1, 8)))
            model_copy(tf.ones((5, 1, 8)))

        mask = tf.cast(tf.random.uniform((5,), minval=0, maxval=2, dtype=tf.int32), dtype=tf.bool)
        reset_states_masked(recurrent_layers, mask)
        reset_states_masked_tf(recurrent_layers_of_copy, mask)

    # store the new states of the models
    reset_states = [tf.identity(s) for s in flatten([l.states for l in model.layers if hasattr(l, "states")])]
    reset_states_copy = [tf.identity(s) for s in flatten([l.states for l in model_copy.layers if hasattr(l, "states")])]

    # assert that the states of the models are equal
    assert np.all([np.all(s == sc) for s, sc in zip(reset_states, reset_states_copy)])

    # assert that the new states differ from the old states
    assert not np.all([np.all(s == sc) for s, sc in zip(states, reset_states)])
    assert not np.all([np.all(s == sc) for s, sc in zip(states_copy, reset_states_copy)])




import json
import re

import matplotlib
import numpy as np
from matplotlib import pyplot as plt
font = {'family' : 'Times',
        'weight' : 'normal',
        'size'   : 12}

matplotlib.rc('font', **font)
with open("./storage/predictability_repeated_august19a.json", "r") as f:
    results = json.load(f)

# remove entries from results that are contain no underscore in their name
results = {k: v for k, v in results.items() if "_" in k}

# create x labels by removing 'activation' potentially surrounded by underscores from the x label name
x_labels = [re.sub(r"(_)?activation(_)?", "", s) for s in results.keys()]
x_labels = [re.sub(r"(_)?internal(_)?", "", s) for s in x_labels]

# rename x label "pmc_recurrent_layer" to "PMC"
x_labels = [re.sub(r"pmc_recurrent_layer", "PMC", s) for s in x_labels]

x = results.keys()
possible_targets = list(list(results.items())[0][1].keys())
print(f"Possible targets: {possible_targets}")

for target in possible_targets:
    fig, ax = plt.subplots()
    ax.bar(x,
       [np.mean(results[s][target]) for s in results.keys()],
       yerr=[np.std(results[s][target]) for s in results.keys()],
       capsize=3)

    ax.set_ylabel("Predictability (R2)")
    ax.set_title(f"{target}")

    ax.set_ylim(0, 1)

    # set labels of the x axis to keys in results
    ax.set_xticks(np.arange(len(x)))
    ax.set_xticklabels(x_labels)

    # show x labels vertically
    plt.xticks(rotation="vertical")

    # set figure size to 8x2 inches
    fig.set_size_inches(12, 2)

    # plt.show()

    # save the figure to a pdf file without whitespace around it
    plt.savefig(f"predictability_{target}.pdf", format="pdf", bbox_inches="tight")


import json
import statistics

import numpy as np
from matplotlib import pyplot as plt

from angorapy.common.const import PATH_TO_EXPERIMENTS

ids = ['1667916362968118', '1667916362368835', '1667916362351224', '1667916362238643', '1667830229410560',
       '1667829693813697', '1667829690921903', '1667829690849747', '1667829690803225', '1667829689265280']

n_workers = []
n_optimizers = []
avg_opt = []
avg_gat = []


for id in ids:
    with open(f"../../../{PATH_TO_EXPERIMENTS}/{id}/meta.json", "r") as f:
        meta = json.load(f)
        n_workers.append(meta["n_cpus"])
        n_optimizers.append(meta["n_gpus"])

    with open(f"../../../{PATH_TO_EXPERIMENTS}/{id}/statistics.json", "r") as f:
        stats = json.load(f)
        avg_opt.append(statistics.mean(stats["optimization_timings"]))
        avg_gat.append(statistics.mean(stats["gathering_timings"]))

zippedi = list(zip(n_workers, n_optimizers, avg_opt, avg_gat))
zippedi.sort()
n_workers, n_optimizers, avg_opt, avg_gat = zip(*zippedi)

fig, ax = plt.subplots()
ax_opt = ax.twiny()

avg_gat = (np.array(avg_gat) - np.min(avg_gat)) / (np.max(avg_gat) - np.min(avg_gat))
avg_opt = (np.array(avg_opt) - np.min(avg_opt)) / (np.max(avg_opt) - np.min(avg_opt))

ax.plot(n_workers, avg_gat)
ax_opt.plot(n_optimizers, avg_opt, color="red")

plt.show()